﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Modbus.Device;
using System.Windows.Forms;
using System.Timers;
using ModbusDemo.windows;
using ModbusDemo;

namespace ModbusDemo.MessageFormat
{
    public static class ModbusComm
    {
        public enum eModbusErrorCode : byte
        {
            MB_ENOERR,                  /*!< no error. */
            MB_ENOREG,                  /*!< illegal register address. */
            MB_EINVAL,                  /*!< illegal argument. */
            MB_EPORTERR,                /*!< porting layer error. */
            MB_ENORES,                  /*!< insufficient resources. */
            MB_EIO,                     /*!< I/O error. */
            MB_EILLSTATE,               /*!< protocol stack in illegal state. */
            MB_ETIMEDOUT,               /*!< timeout error occurred. */
            MB_EMASTERNOERR,            /*!< master mode no error. */
            MB_EMASTERERR,              /*!< master mode error. */
            MB_EMASTERSENT,
            MB_RTUERNOERR
        }
        public static ModbusMaster Modbus;
        public static ushort usCommunFailTimes = 0;
        public static eModbusErrorCode Modbus_ReadHoldingRegistersTask(out ushort[] OutputValue, byte slaveAddress, ushort startAddress, ushort numberOfPoints)
        {
            eModbusErrorCode ModbusErrorCode;
            try
            {
                OutputValue = Modbus.ReadHoldingRegisters(slaveAddress, startAddress, numberOfPoints);
                ModbusErrorCode = eModbusErrorCode.MB_EMASTERNOERR;
            }
            catch (Exception exception)
            {
                //Connection exception
                //No response from server.
                //The server maybe close the com port, or response timeout.
                if (exception.Source.Equals("System"))
                {
                    Console.WriteLine(DateTime.Now.ToString() + " " + exception.Message);
                    Form1.totxt.Log(DateTime.Now.ToString() + " " + exception.Message);
                    OutputValue = new ushort[numberOfPoints];
                    ModbusErrorCode = eModbusErrorCode.MB_ETIMEDOUT;
                    usCommunFailTimes++;
                }
                //The server return error code.
                //You can get the function code and exception code.
                if (exception.Source.Equals("nModbusPC"))
                {
                    string str = exception.Message;
                    int FunctionCode;
                    string ExceptionCode;

                    str = str.Remove(0, str.IndexOf("\r\n") + 17);
                    FunctionCode = Convert.ToInt16(str.Remove(str.IndexOf("\r\n")));
                    Console.WriteLine("Function Code: " + FunctionCode.ToString("X"));
                    Form1.totxt.Log("Function Code: " + FunctionCode.ToString("X"));

                    str = str.Remove(0, str.IndexOf("\r\n") + 17);
                    ExceptionCode = str.Remove(str.IndexOf("-"));
                    switch (ExceptionCode.Trim())
                    {
                        case "1":
                        Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> Illegal function!");
                        Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> Illegal function!");
                        break;
                        case "2":
                        Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> Illegal data address!");
                        Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> Illegal data address!");
                        break;
                        case "3":
                        Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> Illegal data value!");
                        Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> Illegal data value!");
                        break;
                        case "4":
                        Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> Slave device failure!");
                        Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> Slave device failure!");
                        break;
                        case "5":
                        Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> ACKNOWLEDGE!");
                        Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> ACKNOWLEDGE!");
                        break;
                        case "6":
                        Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> SLAVE DEVICE BUSY !");
                        Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> SLAVE DEVICE BUSY !");
                        break;
                        case "8":
                        Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> MEMORY PARITY ERROR !");
                        Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> MEMORY PARITY ERROR !");
                        break;
                        case "A":
                        Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> GATEWAY PATH UNAVAILABLE !");
                        Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> GATEWAY PATH UNAVAILABLE !");
                        break;
                        case "B":
                        Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "---->GATEWAY TARGET DEVICE FAILED TO RESPOND!");
                        Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "---->GATEWAY TARGET DEVICE FAILED TO RESPOND!");
                        break;
                    }
                }
                OutputValue = new ushort[numberOfPoints];
                ModbusErrorCode = eModbusErrorCode.MB_EMASTERERR;
            }
            return ModbusErrorCode;
        }
        public static eModbusErrorCode Modbus_ReadCoilsTask(out bool[] OutputValue, byte slaveAddress, ushort startAddress, ushort numberOfPoints)
        {
            eModbusErrorCode ModbusErrorCode;
            try
            {
                OutputValue = Modbus.ReadCoils(slaveAddress, startAddress, numberOfPoints);
                ModbusErrorCode = eModbusErrorCode.MB_EMASTERNOERR;
            }
            catch (Exception exception)
            {
                //Connection exception
                //No response from server.
                //The server maybe close the com port, or response timeout.
                if (exception.Source.Equals("System"))
                {
                    Console.WriteLine(DateTime.Now.ToString() + " " + exception.Message);
                    Form1.totxt.Log(DateTime.Now.ToString() + " " + exception.Message);
                    OutputValue = new bool[numberOfPoints];
                    ModbusErrorCode = eModbusErrorCode.MB_ETIMEDOUT;
                    usCommunFailTimes++;
                }
                //The server return error code.
                //You can get the function code and exception code.
                if (exception.Source.Equals("nModbusPC"))
                {
                    string str = exception.Message;
                    int FunctionCode;
                    string ExceptionCode;

                    str = str.Remove(0, str.IndexOf("\r\n") + 17);
                    FunctionCode = Convert.ToInt16(str.Remove(str.IndexOf("\r\n")));
                    Console.WriteLine("Function Code: " + FunctionCode.ToString("X"));
                    Form1.totxt.Log("Function Code: " + FunctionCode.ToString("X"));

                    str = str.Remove(0, str.IndexOf("\r\n") + 17);
                    ExceptionCode = str.Remove(str.IndexOf("-"));
                    switch (ExceptionCode.Trim())
                    {
                        case "1":
                        Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> Illegal function!");
                        Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> Illegal function!");
                        break;
                        case "2":
                        Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> Illegal data address!");
                        Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> Illegal data address!");
                        break;
                        case "3":
                        Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> Illegal data value!");
                        Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> Illegal data value!");
                        break;
                        case "4":
                        Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> Slave device failure!");
                        Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> Slave device failure!");
                        break;
                        case "5":
                        Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> ACKNOWLEDGE!");
                        Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> ACKNOWLEDGE!");
                        break;
                        case "6":
                        Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> SLAVE DEVICE BUSY !");
                        Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> SLAVE DEVICE BUSY !");
                        break;
                        case "8":
                        Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> MEMORY PARITY ERROR !");
                        Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> MEMORY PARITY ERROR !");
                        break;
                        case "A":
                        Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> GATEWAY PATH UNAVAILABLE !");
                        Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> GATEWAY PATH UNAVAILABLE !");
                        break;
                        case "B":
                        Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "---->GATEWAY TARGET DEVICE FAILED TO RESPOND!");
                        Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "---->GATEWAY TARGET DEVICE FAILED TO RESPOND!");
                        break;
                    }
                }
                OutputValue = new bool[numberOfPoints];
                ModbusErrorCode = eModbusErrorCode.MB_EMASTERERR;
            }
            return ModbusErrorCode;
        }
        public static eModbusErrorCode Modbus_WriteSingleCoil(byte slaveAddress, ushort startAddress, bool[] value)
        {
            eModbusErrorCode ModbusErrorCode;
            try
            {
                Modbus.WriteMultipleCoils(slaveAddress, startAddress, value);
                ModbusErrorCode = eModbusErrorCode.MB_ENOERR;
            }
            catch (Exception exception)
            {
                //Connection exception
                //No response from server.
                //The server maybe close the com port, or response timeout.
                if (exception.Source.Equals("System"))
                {
                    Console.WriteLine(DateTime.Now.ToString() + " " + exception.Message);
                    Form1.totxt.Log(DateTime.Now.ToString() + " " + exception.Message);
                    ModbusErrorCode = eModbusErrorCode.MB_ETIMEDOUT;
                }
                //The server return error code.
                //You can get the function code and exception code.
                if (exception.Source.Equals("nModbusPC"))
                {
                    string str = exception.Message;
                    int FunctionCode;
                    string ExceptionCode;

                    str = str.Remove(0, str.IndexOf("\r\n") + 17);
                    FunctionCode = Convert.ToInt16(str.Remove(str.IndexOf("\r\n")));
                    Console.WriteLine("Function Code: " + FunctionCode.ToString("X"));
                    Form1.totxt.Log("Function Code: " + FunctionCode.ToString("X"));

                    str = str.Remove(0, str.IndexOf("\r\n") + 17);
                    ExceptionCode = str.Remove(str.IndexOf("-"));
                    switch (ExceptionCode.Trim())
                    {
                        case "1":
                        Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> Illegal function!");
                        Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> Illegal function!");
                        break;
                        case "2":
                        Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> Illegal data address!");
                        Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> Illegal data address!");
                        break;
                        case "3":
                        Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> Illegal data value!");
                        Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> Illegal data value!");
                        break;
                        case "4":
                        Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> Slave device failure!");
                        Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> Slave device failure!");
                        break;
                        case "5":
                        Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> ACKNOWLEDGE!");
                        Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> ACKNOWLEDGE!");
                        break;
                        case "6":
                        Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> SLAVE DEVICE BUSY !");
                        Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> SLAVE DEVICE BUSY !");
                        break;
                        case "8":
                        Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> MEMORY PARITY ERROR !");
                        Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> MEMORY PARITY ERROR !");
                        break;
                        case "A":
                        Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> GATEWAY PATH UNAVAILABLE !");
                        Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> GATEWAY PATH UNAVAILABLE !");
                        break;
                        case "B":
                        Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "---->GATEWAY TARGET DEVICE FAILED TO RESPOND!");
                        Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "---->GATEWAY TARGET DEVICE FAILED TO RESPOND!");
                        break;
                    }
                }
                ModbusErrorCode = eModbusErrorCode.MB_EMASTERERR;
            }
            return ModbusErrorCode;
        }
        public static eModbusErrorCode Modbus_WriteMultipleCoils(byte slaveAddress, ushort startAddress, bool[] data)
        {
            eModbusErrorCode ModbusErrorCode;
            try
            {
                Modbus.WriteMultipleCoils(slaveAddress, startAddress, data);
                ModbusErrorCode = eModbusErrorCode.MB_ENOERR;
            }
            catch (Exception exception)
            {
                //Connection exception
                //No response from server.
                //The server maybe close the com port, or response timeout.
                if (exception.Source.Equals("System"))
                {
                    Console.WriteLine(DateTime.Now.ToString() + " " + exception.Message);
                    Form1.totxt.Log(DateTime.Now.ToString() + " " + exception.Message);
                    ModbusErrorCode = eModbusErrorCode.MB_ETIMEDOUT;
                }
                //The server return error code.
                //You can get the function code and exception code.
                if (exception.Source.Equals("nModbusPC"))
                {
                    string str = exception.Message;
                    int FunctionCode;
                    string ExceptionCode;

                    str = str.Remove(0, str.IndexOf("\r\n") + 17);
                    FunctionCode = Convert.ToInt16(str.Remove(str.IndexOf("\r\n")));
                    Console.WriteLine("Function Code: " + FunctionCode.ToString("X"));
                    Form1.totxt.Log("Function Code: " + FunctionCode.ToString("X"));

                    str = str.Remove(0, str.IndexOf("\r\n") + 17);
                    ExceptionCode = str.Remove(str.IndexOf("-"));
                    switch (ExceptionCode.Trim())
                    {
                        case "1":
                        Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> Illegal function!");
                        Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> Illegal function!");
                        break;
                        case "2":
                        Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> Illegal data address!");
                        Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> Illegal data address!");
                        break;
                        case "3":
                        Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> Illegal data value!");
                        Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> Illegal data value!");
                        break;
                        case "4":
                        Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> Slave device failure!");
                        Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> Slave device failure!");
                        break;
                        case "5":
                        Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> ACKNOWLEDGE!");
                        Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> ACKNOWLEDGE!");
                        break;
                        case "6":
                        Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> SLAVE DEVICE BUSY !");
                        Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> SLAVE DEVICE BUSY !");
                        break;
                        case "8":
                        Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> MEMORY PARITY ERROR !");
                        Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> MEMORY PARITY ERROR !");
                        break;
                        case "A":
                        Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "----> GATEWAY PATH UNAVAILABLE !");
                        Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "----> GATEWAY PATH UNAVAILABLE !");
                        break;
                        case "B":
                        Console.WriteLine("Exception Code: " + ExceptionCode.Trim() + "---->GATEWAY TARGET DEVICE FAILED TO RESPOND!");
                        Form1.totxt.Log("Exception Code: " + ExceptionCode.Trim() + "---->GATEWAY TARGET DEVICE FAILED TO RESPOND!");
                        break;
                    }
                }
                ModbusErrorCode = eModbusErrorCode.MB_EMASTERERR;
            }
            return ModbusErrorCode;
        }
        //private void Init_Read_data()
        //{



        //}
        //private async void Modbus_polling()
        //{
        //    float EastTemperature = 0;
        //    float Easternwaterlevel = 0;
        //    float WestWaterLevel = 0;
        //ushort[] temperature = { };
        //ushort[] type = { };
        //string[] vs = { };
        //string[] s = { };
        //string[] s6 = { };
        //ushort[] register = { };
        //ushort[] Inputtype = { };
        //ushort[] registerN = { };
        //ushort[] InputtypeN = { };
        //bool[] EIO = { };
        //bool[] SIO = { };
        //bool[] WIO = { };
        //int[] EOnOff = { 0 };
        //int[] SOnOff = { 0 };
        //int[] WOnOff = { 0 };
        //pcDeviceAlarm pcDeviceAlarmdata = new pcDeviceAlarm();
        //Modbus_ReadHoldingRegistersTask(out temperature, 1, 0, 4, out int signT);
        //Modbus_ReadHoldingRegistersTask(out type, 1, 10, 4, out int sign);
        //UInt64 time_1 = Form1.GetTimeStamp();
        //Modbus_ReadHoldingRegistersTask(out register, 3, 0, 8, out int signr);
        //Modbus_ReadHoldingRegistersTask(out Inputtype, 3, 10, 8, out int signI);
        //UInt64 time_2 = Form1.GetTimeStamp();
        //Modbus_ReadHoldingRegistersTask(out registerN, 6, 0, 8, out int signre);
        //Modbus_ReadHoldingRegistersTask(out InputtypeN, 6, 10, 8, out int signIn);
        //UInt64 time_3 = Form1.GetTimeStamp();
        //Modbus_ReadCoilsTask(out EIO, 5, 16, 6, out int signE);

        //Modbus_ReadCoilsTask(out SIO, 8, 16, 6, out int signS);

        //Modbus_ReadCoilsTask(out WIO, 4, 16, 6, out int signW);
        //if (sign == -1)
        //{
        //    ushort bAddress = 1;
        //    ushort[] bChannel = new ushort[pcDeviceAlarm.usChannelNumber];
        //    pcDeviceAlarm.eAlarmCode[] eAlarmCode = new pcDeviceAlarm.eAlarmCode[pcDeviceAlarm.usChannelNumber];
        //    for (ushort i = 0; i < pcDeviceAlarm.usChannelNumber; i++)
        //    {
        //        eAlarmCode[i] = pcDeviceAlarm.eAlarmCode.CommunError;
        //    }
        //    pcDeviceAlarmdata.vSetAlarmList(bAddress, bChannel, eAlarmCode);
        //    await pcDeviceAlarmdata.vPublishAlarmPackageJson(mqttClient);
        //}
        //if (sign == -1 || signT ==-1|| signW ==-1 || signr == -1 || signI == -1 || signre == -1 || signIn == -1 || signE == -1 || signS == -1)
        //{
        //    vs = null;
        //    type = null;
        //    EastTemperature = 0;
        //    Easternwaterlevel = 0;
        //    WestWaterLevel = 0;
        //}
        //else
        //{
        //vs = rtd.RtdValue(temperature, type);
        //EastTemperature = float.Parse(vs[0]);
        //s = Analog.Value(register, Inputtype);
        //Easternwaterlevel = (float.Parse(s[0]) - 4) / 16 * 6;
        //s6 = Analog.Value(registerN, InputtypeN);
        //WestWaterLevel = (float.Parse(s6[0]) - 4) / 16 * 6;
        //label3.Text = EastTemperature.ToString();
        //label4.Text = Easternwaterlevel.ToString();
        //EOnOff = state.IO(EIO);////东中转池   
        //SOnOff = state.IO(SIO);////西中转池
        //WOnOff = state.IO(WIO); ////东南北水
        //}



        //WTDR14P x = new WTDR14P("1", EastTemperature, 0, 0, 0, time_1);
        //WTDR18X p = new WTDR18X("3", Easternwaterlevel, 0, 0, 0, 0, 0, 0, 0, time_2);
        //WTDR18X q = new WTDR18X("6", WestWaterLevel, 0, 0, 0, 0, 0, 0, 0, time_3);

        ////东搅拌
        //WTDR66C EastStir = new WTDR66C("5", EOnOff[0], EOnOff[1], EOnOff[2], EOnOff[3], EOnOff[4], EOnOff[5], Form1.GetTimeStamp());
        ////东南北水
        //WTDR66C WestStir = new WTDR66C("4", WOnOff[0], WOnOff[1], WOnOff[2], WOnOff[3], WOnOff[4], WOnOff[5], Form1.GetTimeStamp());
        ////西搅拌
        //WTDR66C SouthWater = new WTDR66C("8", SOnOff[0], SOnOff[1], SOnOff[2], SOnOff[3], SOnOff[4], SOnOff[5], Form1.GetTimeStamp());

        //    mqttData = "[" +
        //        JsonConvert.SerializeObject(x) + "," +
        //         JsonConvert.SerializeObject(p) + "," +
        //         JsonConvert.SerializeObject(q) + "," +
        //         JsonConvert.SerializeObject(EastStir) + "," +
        //          JsonConvert.SerializeObject(WestStir) + "," +
        //         JsonConvert.SerializeObject(SouthWater)
        //        + "]";
        //}
    }
}
